﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using HeuristicLab.Data;
using HeuristicLab.Common;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;
using HeuristicLab.Core;

namespace HeuristicLab.Problems.MetaOptimization {
  [StorableClass]
  public class PercentValueRange : Range<PercentValue> {

    public PercentValueRange(PercentValue lowerBound, PercentValue upperBound, PercentValue stepSize) : base(lowerBound, upperBound, stepSize) { }
    public PercentValueRange() { }
    [StorableConstructor]
    protected PercentValueRange(bool deserializing) : base(deserializing) { }
    protected PercentValueRange(PercentValueRange original, Cloner cloner) : base(original, cloner) { }
    public override IDeepCloneable Clone(Cloner cloner) {
      return new PercentValueRange(this, cloner);
    }

    protected override PercentValue GetRandomSample(IRandom random) {
      double val;
      do {
        val = Math.Round((random.NextDouble() * (UpperBound.Value - LowerBound.Value) + LowerBound.Value) / StepSize.Value, 0) * StepSize.Value;
      } while (val < LowerBound.Value || val > UpperBound.Value);
      return new PercentValue(val);
    }

    public DoubleValueRange AsDoubleValueRange() {
      return new DoubleValueRange(LowerBound, UpperBound, StepSize);
    }
  }
}
